@@ -7,7 +7,7 @@ Attean::API::Binding - Name to term bindings
 
 =head1 VERSION
 
-This document describes Attean::API::Binding version 0.007
+This document describes Attean::API::Binding version 0.008
 
 =head1 DESCRIPTION
 
@@ -71,7 +71,7 @@ otherwise.
 
 use Type::Tiny::Role;
 
-package Attean::API::Binding 0.007 {
+package Attean::API::Binding 0.008 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed);
 	use List::MoreUtils qw(zip);
@@ -148,7 +148,7 @@ package Attean::API::Binding 0.007 {
 	}
 }
 
-package Attean::API::TripleOrQuadPattern 0.007 {
+package Attean::API::TripleOrQuadPattern 0.008 {
 	use Scalar::Util qw(blessed);
 	use Attean::RDF;
 	use Moo::Role;
@@ -215,12 +215,12 @@ package Attean::API::TripleOrQuadPattern 0.007 {
 	}
 }
 
-package Attean::API::TripleOrQuad 0.007 {
+package Attean::API::TripleOrQuad 0.008 {
 	use Moo::Role;
 	with 'Attean::API::TripleOrQuadPattern';
 }
 
-package Attean::API::TriplePattern 0.007 {
+package Attean::API::TriplePattern 0.008 {
 	use Moo::Role;
 	use List::MoreUtils qw(zip);
 	use Scalar::Util qw(blessed);
@@ -268,7 +268,7 @@ package Attean::API::TriplePattern 0.007 {
 	with 'Attean::API::TripleOrQuadPattern', 'Attean::API::Binding';
 }
 
-package Attean::API::Triple 0.007 {
+package Attean::API::Triple 0.008 {
 	use Moo::Role;
 	
 	if ($ENV{ATTEAN_TYPECHECK}) {
@@ -303,7 +303,7 @@ package Attean::API::Triple 0.007 {
 	with 'Attean::API::TriplePattern', 'Attean::API::TripleOrQuad', 'Attean::API::Binding';
 }
 
-package Attean::API::QuadPattern 0.007 {
+package Attean::API::QuadPattern 0.008 {
 	use Scalar::Util qw(blessed);
 	use Moo::Role;
 	
@@ -336,7 +336,7 @@ package Attean::API::QuadPattern 0.007 {
 	with 'Attean::API::TripleOrQuadPattern', 'Attean::API::Binding';
 }
 
-package Attean::API::Quad 0.007 {
+package Attean::API::Quad 0.008 {
 	use Moo::Role;
 	
 	if ($ENV{ATTEAN_TYPECHECK}) {
@@ -363,12 +363,22 @@ package Attean::API::Quad 0.007 {
 }
 
 
-package Attean::API::Result 0.007 {
+package Attean::API::Result 0.008 {
 	use Moo::Role;
 	use Scalar::Util qw(refaddr);
+	use Types::Standard qw(HashRef);
 	use namespace::clean;
 	
-	sub join {
+	has 'eval_stash' => (is => 'rw', isa => HashRef);
+
+	sub BUILD {
+		my $self	= shift;
+		if (not $self->eval_stash) {
+			$self->eval_stash({});
+		}
+	}
+	
+	sub shared_domain {
 		my $self	= shift;
 		my $class	= ref($self);
 		my $rowb	= shift;
@@ -377,6 +387,14 @@ package Attean::API::Result 0.007 {
 		my @keysa	= $self->variables;
 		@keysa{ @keysa }	= (1) x scalar(@keysa);
 		my @shared	= grep { exists $keysa{ $_ } } ($rowb->variables);
+		return @shared;
+	}
+	
+	sub join {
+		my $self	= shift;
+		my $class	= ref($self);
+		my $rowb	= shift;
+		my @shared	= $self->shared_domain($rowb);
 		foreach my $key (@shared) {
 			my $val_a	= $self->value($key);
 			my $val_b	= $rowb->value($key);
