@@ -7,7 +7,7 @@ Attean::Blank - RDF blank nodes
 
 =head1 VERSION
 
-This document describes Attean::Blank version 0.005
+This document describes Attean::Blank version 0.008
 
 =head1 SYNOPSIS
 
@@ -33,7 +33,7 @@ This role consumes L<Attean::API::Blank>, which provides the following methods:
 
 =cut
 
-package Attean::Blank 0.005 {
+package Attean::Blank 0.008 {
 	use Moo;
 	use Types::Standard qw(Str);
 	use Data::UUID;
@@ -52,7 +52,8 @@ package Attean::Blank 0.005 {
 			my $uuid	= Data::UUID->new->create_hex;
 			return $class->$orig(value => $uuid);
 		} elsif (scalar(@_) == 1) {
-			return $class->$orig(value => shift);
+			my $value	= shift // '';
+			return $class->$orig(value => $value);
 		}
 		return $class->$orig(@_);
 	};
