@@ -7,7 +7,7 @@ Attean::Expression - SPARQL Expressions
 
 =head1 VERSION
 
-This document describes Attean::Expression version 0.004_01
+This document describes Attean::Expression version 0.008
 
 =head1 SYNOPSIS
 
@@ -40,7 +40,7 @@ use Attean::API::Expression;
 
 =cut
 
-package Attean::ValueExpression 0.004 {
+package Attean::ValueExpression 0.008 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf);
 	use namespace::clean;
@@ -53,6 +53,8 @@ package Attean::ValueExpression 0.004 {
 	}
 	has 'value' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable']);
 	
+	sub tree_attributes { return qw(operator) }
+
 	sub is_stable {
 		return 1;
 	}
@@ -86,7 +88,7 @@ package Attean::ValueExpression 0.004 {
 
 =cut
 
-package Attean::UnaryExpression 0.004 {
+package Attean::UnaryExpression 0.008 {
 	use Moo;
 	use Types::Standard qw(Enum);
 	use namespace::clean;
@@ -106,6 +108,8 @@ package Attean::UnaryExpression 0.004 {
 		$type->assert_valid($self->operator);
 	}
 	
+	sub tree_attributes { return qw(operator) }
+
 	sub is_stable {
 		my $self	= shift;
 		foreach my $c (@{ $self->children }) {
@@ -121,7 +125,7 @@ package Attean::UnaryExpression 0.004 {
 
 =cut
 
-package Attean::BinaryExpression 0.004 {
+package Attean::BinaryExpression 0.008 {
 	use Moo;
 	use Types::Standard qw(Enum);
 	use namespace::clean;
@@ -132,6 +136,8 @@ package Attean::BinaryExpression 0.004 {
 		$type->assert_valid($self->operator);
 	}
 	
+	sub tree_attributes { return qw(operator) }
+
 	sub is_stable {
 		my $self	= shift;
 		foreach my $c (@{ $self->children }) {
@@ -147,7 +153,7 @@ package Attean::BinaryExpression 0.004 {
 
 =cut
 
-package Attean::FunctionExpression 0.004 {
+package Attean::FunctionExpression 0.008 {
 	use Moo;
 	use Types::Standard qw(Enum ConsumerOf HashRef);
 	use Types::Common::String qw(UpperCaseStr);
@@ -182,6 +188,8 @@ package Attean::FunctionExpression 0.004 {
 		}
 	}
 
+	sub tree_attributes { return qw(operator) }
+
 	sub is_stable {
 		my $self	= shift;
 		return 0 if ($self->operator =~ m/^(?:RAND|BNODE|UUID|STRUUID|NOW)$/);
@@ -192,7 +200,7 @@ package Attean::FunctionExpression 0.004 {
 	}
 }
 
-package Attean::AggregateExpression 0.004 {
+package Attean::AggregateExpression 0.008 {
 	use Moo;
 	use Types::Standard qw(Bool Enum Str HashRef ConsumerOf);
 	use Types::Common::String qw(UpperCaseStr);
@@ -214,9 +222,19 @@ package Attean::AggregateExpression 0.004 {
 	has 'distinct'		=> (is => 'ro', isa => Bool, default => 0);
 	has 'variable'		=> (is => 'ro', isa => ConsumerOf['Attean::API::Variable'], required => 1);
 	with 'Attean::API::AggregateExpression';
+
+	sub tree_attributes { return qw(operator scalar_vars variable) }
+
+	sub is_stable {
+		my $self	= shift;
+		foreach my $expr (@{ $self->groups }, values %{ $self->aggregates }) {
+			return 0 unless ($expr->is_stable);
+		}
+		return 1;
+	}
 }
 
-package Attean::CastExpression 0.004 {
+package Attean::CastExpression 0.008 {
 	use Moo;
 	use Types::Standard qw(Enum ConsumerOf);
 	use namespace::clean;
@@ -232,6 +250,8 @@ package Attean::CastExpression 0.004 {
 		$type->assert_valid($self->datatype->value);
 	}
 	
+	sub tree_attributes { return qw(operator datatype) }
+
 	sub is_stable {
 		my $self	= shift;
 		foreach my $c (@{ $self->children }) {
@@ -243,7 +263,7 @@ package Attean::CastExpression 0.004 {
 	with 'Attean::API::UnaryExpression', 'Attean::API::Expression', 'Attean::API::UnaryQueryTree';
 }
 
-package Attean::ExistsExpression 0.004 {
+package Attean::ExistsExpression 0.008 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf);
 	use namespace::clean;
@@ -273,6 +293,46 @@ package Attean::ExistsExpression 0.004 {
 		return "EXISTS " . $self->pattern->as_sparql( level => $level+1, indent => $sp );
 	}
 
+	sub tree_attributes { return qw(operator pattern) }
+
+	sub is_stable {
+		my $self	= shift;
+		# TODO: need deep analysis of exists pattern to tell if this is stable
+		# (there might be an unstable filter expression deep inside the pattern)
+		return 0;
+	}
+}
+
+package Attean::ExistsPlanExpression 0.008 {
+	use Moo;
+	use Types::Standard qw(ConsumerOf);
+	use namespace::clean;
+
+	with 'Attean::API::Expression';
+	sub arity { return 0 }
+	sub BUILDARGS {
+		my $class	= shift;
+		return $class->SUPER::BUILDARGS(@_, operator => '_existsplan');
+	}
+	has 'plan' => (is => 'ro', isa => ConsumerOf['Attean::API::BindingSubstitutionPlan']);
+	sub as_string {
+		my $self	= shift;
+		# TODO: implement as_string for EXISTS patterns
+		return "Attean::ExistsPlanExpression { ... }";
+	}
+	sub as_sparql {
+		my $self	= shift;
+		my %args	= @_;
+		my $level	= $args{level} // 0;
+		my $sp		= $args{indent} // '    ';
+		my $indent	= $sp x $level;
+
+		# TODO: implement as_string for EXISTS patterns
+		return "EXISTS { " . $self->pattern->as_sparql( level => $level+1, indent => $sp ) . " }";
+	}
+
+	sub tree_attributes { return qw(operator plan) }
+
 	sub is_stable {
 		my $self	= shift;
 		# TODO: need deep analysis of exists pattern to tell if this is stable
