@@ -7,7 +7,7 @@ Attean::IDPQueryPlanner - Iterative dynamic programming query planner
 
 =head1 VERSION
 
-This document describes Attean::IDPQueryPlanner version 0.007
+This document describes Attean::IDPQueryPlanner version 0.008
 
 =head1 SYNOPSIS
 
@@ -34,13 +34,14 @@ use Attean::Algebra;
 use Attean::Plan;
 use Attean::Expression;
 
-package Attean::IDPQueryPlanner 0.007 {
+package Attean::IDPQueryPlanner 0.008 {
 	use Moo;
 	use Encode qw(encode);
 	use Attean::RDF;
 	use LWP::UserAgent;
 	use Scalar::Util qw(blessed reftype);
-	use List::Util qw(all any reduce);
+	use List::Util qw(reduce);
+	use List::MoreUtils qw(all any);
 	use Types::Standard qw(Int ConsumerOf InstanceOf);
 	use URI::Escape;
 	use Algorithm::Combinatorics qw(subsets);
@@ -152,6 +153,26 @@ the supplied C<< $active_graph >>.
 		} elsif ($algebra->isa('Attean::Algebra::Filter')) {
 			# TODO: simple range relation filters can be handled differently if that filter operates on a variable that is part of the ordering
 			my $expr	= $algebra->expression;
+			my $w	= Attean::TreeRewriter->new(types => ['Attean::API::DirectedAcyclicGraph']);
+			$w->register_pre_handler(sub {
+				my ($t, $parent, $thunk)	= @_;
+				if ($t->isa('Attean::ExistsExpression')) {
+					my $pattern	= $t->pattern;
+					my $plan	= $self->plan_for_algebra($pattern, $model, $active_graphs, $default_graphs, @_);
+					unless ($plan->does('Attean::API::BindingSubstitutionPlan')) {
+						die 'Exists plan does not consume Attean::API::BindingSubstitutionPlan: ' . $plan->as_string;
+					}
+					my $new	= Attean::ExistsPlanExpression->new(
+						plan => $plan,
+					);
+					return (1, 0, $new);
+				}
+				return (0, 1, $t);
+			});
+			my ($changed, $rewritten)	= $w->rewrite($expr, {});
+			if ($changed) {
+				$expr	= $rewritten;
+			}
 			
 			my $var		= $self->new_temporary('filter');
 			my %exprs	= ($var => $expr);
@@ -268,35 +289,6 @@ the supplied C<< $active_graph >>.
 		} elsif ($algebra->isa('Attean::Algebra::Slice')) {
 			my $limit	= $algebra->limit;
 			my $offset	= $algebra->offset;
-			if (my $child = $algebra->unary) {
-				my $vars;
-				if ($child->isa('Attean::Algebra::Project')) {
-					$vars		= $child->variables;
-					$child		= $child->unary;
-				}
-				
-				if ($child->isa('Attean::Algebra::OrderBy')) {
-					my $count	= $limit + $offset;
-					my @cmps	= @{ $child->comparators };
-					my ($exprs, $ascending, $svars)	= $self->_order_by($child);
-					my @plans;
-					foreach my $plan ($self->plans_for_algebra($child->children->[0], $model, $active_graphs, $default_graphs, @_)) {
-						my @vars	= @{ $plan->in_scope_variables };
-						my @evars	= (@vars, keys %$exprs);
-						my $extend	= Attean::Plan::Extend->new(children => [$plan], expressions => $exprs, distinct => 0, ordered => $plan->ordered);
-						my $ordered	= Attean::Plan::HeapSort->new(children => [$extend], limit => $count, variables => $svars, ascending => $ascending, distinct => $plan->distinct, ordered => \@cmps);
-						my $proj	= $self->new_projection($ordered, $plan->distinct, @{ $plan->in_scope_variables });
-						if ($offset) {
-							$proj	= Attean::Plan::Slice->new(children => [$proj], offset => $offset, distinct => $proj->distinct, ordered => $proj->ordered);
-						}
-						push(@plans, $proj);
-					}
-					return @plans;
-				}
-			}
-			
-			
-			
 			my @plans;
 			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
 				my $vars	= $plan->in_scope_variables;
@@ -325,17 +317,194 @@ the supplied C<< $active_graph >>.
 				push(@plans, $extend);
 			}
 			return @plans;
+		} elsif ($algebra->isa('Attean::Algebra::Group')) {
+			my $aggs	= $algebra->aggregates;
+			my $groups	= $algebra->groupby;
+			my %exprs;
+			foreach my $expr (@$aggs) {
+				my $var	= $expr->variable->value;
+				$exprs{$var}	= $expr;
+			}
+			my @plans;
+			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
+				my $extend		= Attean::Plan::Aggregate->new(children => [$plan], aggregates => \%exprs, groups => $groups, distinct => 0, ordered => []);
+				push(@plans, $extend);
+			}
+			return @plans;
 		} elsif ($algebra->isa('Attean::Algebra::Ask')) {
 			my @plans;
 			foreach my $plan ($self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_)) {
 				return Attean::Plan::Exists->new(children => [$plan], distinct => 1, ordered => []);
 			}
 			return @plans;
-		} elsif ($algebra->isa('Attean::Algebra::Group')) {
 		} elsif ($algebra->isa('Attean::Algebra::Path')) {
+			my $s		= $algebra->subject;
+			my $path	= $algebra->path;
+			my $o		= $algebra->object;
+			
+			my @algebra	= $self->simplify_path($s, $path, $o);
+			
+			my @join;
+			if (scalar(@algebra)) {
+				my @triples;
+				while (my $pa = shift(@algebra)) {
+					if ($pa->isa('Attean::TriplePattern')) {
+						push(@triples, $pa);
+					} else {
+						if (scalar(@triples)) {
+							push(@join, Attean::Algebra::BGP->new( triples => [@triples] ));
+							@triples	= ();
+						}
+						push(@join, $pa);
+					}
+				}
+				if (scalar(@triples)) {
+					push(@join, Attean::Algebra::BGP->new( triples => [@triples] ));
+				}
+				
+				my @vars	= $algebra->in_scope_variables;
+				
+				my @joins	= $self->group_join_plans($model, $active_graphs, $default_graphs, $interesting, map {
+					[$self->plans_for_algebra($_, $model, $active_graphs, $default_graphs, @_)]
+				} @join);
+				
+				my @plans;
+				foreach my $j (@joins) {
+					push(@plans, Attean::Plan::Project->new(children => [$j], variables => [map { Attean::Variable->new($_) } @vars], distinct => 0, ordered => []));
+				}
+				return @plans;
+				
+			} elsif ($path->isa('Attean::Algebra::ZeroOrMorePath') or $path->isa('Attean::Algebra::OneOrMorePath')) {
+				my $skip	= $path->isa('Attean::Algebra::OneOrMorePath') ? 1 : 0;
+				my $begin	= variable($self->new_temporary('pp'));
+				my $end		= variable($self->new_temporary('pp'));
+				my $s_var	= $s->does('Attean::API::Variable');
+				my $o_var	= $o->does('Attean::API::Variable');
+				
+				my $child	= $path->children->[0];
+				my $a;
+				if ($s_var and not($o_var)) {
+					my $inv	= Attean::Algebra::InversePath->new( children => [$child] );
+					$a		= Attean::Algebra::Path->new( subject => $end, path => $inv, object => $begin );
+				} else {
+					$a		= Attean::Algebra::Path->new( subject => $begin, path => $child, object => $end );
+				}
+				my @cplans	= $self->plans_for_algebra($a, $model, $active_graphs, $default_graphs, @_);
+				my @plans;
+				foreach my $cplan (@cplans) {
+					my $plan	= Attean::Plan::ALPPath->new(
+						subject => $s,
+						children => [$cplan],
+						object => $o,
+						graph => $active_graphs,
+						skip => $skip,
+						step_begin => $begin,
+						step_end => $end,
+						distinct => 0,
+						ordered => []
+					);
+					push(@plans, $plan);
+				}
+				return @plans;
+			} elsif ($path->isa('Attean::Algebra::ZeroOrOnePath')) {
+				my $a		= Attean::Algebra::Path->new( subject => $s, path => $path->children->[0], object => $o );
+				my @children	= $self->plans_for_algebra($a, $model, $active_graphs, $default_graphs, @_);
+				my @plans;
+				foreach my $plan (@children) {
+					push(@plans, Attean::Plan::ZeroOrOnePath->new(
+						subject => $s,
+						children => [$plan],
+						object => $o,
+						graph => $active_graphs,
+						distinct => 0,
+						ordered => []
+					));
+				}
+				return @plans;
+			} else {
+				die "Cannot simplify property path $path: " . $algebra->as_string;
+			}
 		} elsif ($algebra->isa('Attean::Algebra::Construct')) {
+			my @children	= $self->plans_for_algebra($child, $model, $active_graphs, $default_graphs, @_);
+			my @plans;
+			foreach my $plan (@children) {
+				push(@plans, Attean::Plan::Construct->new(triples => $algebra->triples, children => [$plan], distinct => 0, ordered => []));
+			}
+			return @plans;
+		} elsif ($algebra->isa('Attean::Algebra::Group')) {
+		}
+		die "Unimplemented algebra evaluation for: " . $algebra->as_string;
+	}
+	
+# 	sub plans_for_unbounded_path {
+# 		my $self			= shift;
+# 		my $algebra			= shift;
+# 		my $model			= shift;
+# 		my $active_graphs	= shift;
+# 		my $default_graphs	= shift;
+# 		my %args			= @_;
+# 		
+# 		my $s		= $algebra->subject;
+# 		my $path	= $algebra->path;
+# 		my $o		= $algebra->object;
+# 		
+# 		return Attean::Plan::ALPPath->new(distinct => 0, ordered => []);
+# 	}
+	
+	sub _package {
+		my $self	= shift;
+		my @args	= @_;
+		
+		my @bgptriples	= map { @{ $_->triples } } grep { $_->isa('Attean::Algebra::BGP') } @args;
+		my @triples		= grep { $_->isa('Attean::TriplePattern') } @args;
+		my @rest		= grep { not $_->isa('Attean::Algebra::BGP') and not $_->isa('Attean::TriplePattern') } @args;
+		if (scalar(@rest) == 0) {
+			return Attean::Algebra::BGP->new( triples => [@bgptriples, @triples] );
+		} else {
+			my $p	= Attean::Algebra::BGP->new( triples => [@bgptriples, @triples] );
+			while (scalar(@rest) > 0) {
+				$p	= Attean::Algebra::Join->new( children => [$p, shift(@rest)] );
+			}
+			return $p;
+		}
+	}
+	
+	sub simplify_path {
+		my $self	= shift;
+		my $s		= shift;
+		my $path	= shift;
+		my $o		= shift;
+		if ($path->isa('Attean::Algebra::SequencePath')) {
+			my $jvar		= variable($self->new_temporary('pp'));
+			my ($lhs, $rhs)	= @{ $path->children };
+			my @paths;
+			push(@paths, $self->simplify_path($s, $lhs, $jvar));
+			push(@paths, $self->simplify_path($jvar, $rhs, $o));
+			return $self->_package(@paths);
+		} elsif ($path->isa('Attean::Algebra::InversePath')) {
+			my ($ipath)	= @{ $path->children };
+			return $self->simplify_path($o, $ipath, $s);
+		} elsif ($path->isa('Attean::Algebra::PredicatePath')) {
+			my $pred	= $path->predicate;
+			return Attean::TriplePattern->new($s, $pred, $o);
+		} elsif ($path->isa('Attean::Algebra::AlternativePath')) {
+			my ($l, $r)	= @{ $path->children };
+			my $la	= $self->_package($self->simplify_path($s, $l, $o));
+			my $ra	= $self->_package($self->simplify_path($s, $r, $o));
+			return Attean::Algebra::Union->new( children => [$la, $ra] );
+		} elsif ($path->isa('Attean::Algebra::NegatedPropertySet')) {
+			my @preds	= @{ $path->predicates };
+			my $pvar	= variable($self->new_temporary('nps'));
+			my $pvar_e	= Attean::ValueExpression->new( value => $pvar );
+			my $t		= Attean::TriplePattern->new($s, $pvar, $o);
+			my @vals	= map { Attean::ValueExpression->new( value => $_ ) } @preds;
+			my $expr	= Attean::FunctionExpression->new( children => [$pvar_e, @vals], operator => 'notin' );
+			my $bgp		= Attean::Algebra::BGP->new( triples => [$t] );
+			my $f		= Attean::Algebra::Filter->new( children => [$bgp], expression => $expr );
+			return $f;
+		} else {
+			return;
 		}
-		die "Unimplemented algebra evaluation for: $algebra";
 	}
 	
 	sub new_projection {
@@ -617,10 +786,8 @@ sub-plan participating in the join.
 						push(@plans, Attean::Plan::HashJoin->new(children => [$lhs, $rhs], left => 1, expression => $expr, join_variables => \@join_vars, distinct => 0, ordered => []));
 					}
 				} elsif ($minus) {
+					# we can't use a hash join for MINUS queries, because of the definition of MINUS having a special case for compatible results that have disjoint domains
 					push(@plans, Attean::Plan::NestedLoopJoin->new(children => [$lhs, $rhs], anti => 1, join_variables => \@join_vars, distinct => 0, ordered => $lhs->ordered));
-					if (scalar(@join_vars) > 0) {
-						push(@plans, Attean::Plan::HashJoin->new(children => [$lhs, $rhs], anti => 1, join_variables => \@join_vars, join_variables => \@join_vars, distinct => 0, ordered => []));
-					}
 				} else {
 					# nested loop joins work in all cases
 					push(@plans, Attean::Plan::NestedLoopJoin->new(children => [$lhs, $rhs], join_variables => \@join_vars, distinct => 0, ordered => $lhs->ordered));
